local addonName = ...
local db
local itemLink, bag, slot
local Cache = {}
local tooltipName = "APUscanner"
local tooltipScanner = CreateFrame("GameTooltip", tooltipName, nil, "GameTooltipTemplate")
local ARTIFACT_POWER = ARTIFACT_POWER

local function ScanBags()
	for bag = 0, NUM_BAG_SLOTS do
		for slot = 1, GetContainerNumSlots(bag) do
			local itemLink = GetContainerItemLink(bag, slot)
			if itemLink then
				if Cache[itemLink] then
					return itemLink, bag, slot
				else
					tooltipScanner:SetOwner(UIParent, "ANCHOR_NONE")
					tooltipScanner:SetHyperlink(itemLink)
					local tooltipText = _G[tooltipName.."TextLeft2"]:GetText()
					if tooltipText and tooltipText:match(ARTIFACT_POWER) then
						Cache[itemLink] = true
						return itemLink, bag, slot
					end
				end
			end
		end
	end
	return nil
end

local function Update()
	local self = ArtifactPowerUserButton
	if not InCombatLockdown() and not db.hide then
		itemLink, bag, slot = ScanBags()
		if itemLink then
			self:SetAttribute("type", "item")
			self:SetAttribute("item", bag.." "..slot)
			itemTexture = GetItemIcon(itemLink)
			self.icon:SetTexture(itemTexture)
			self:Show()
		else
			self:Hide()
		end
	else
		self:Hide()
	end
end

local function LoadSettings(self)
	if ArtifactPowerUserDB == nil then 
		ArtifactPowerUserDB = {}
	end
	db = ArtifactPowerUserDB
	if db.hide == nil then db.hide = false end
	if db.size == nil then db.size = 64 end
	self:SetSize(db.size, db.size)
	if db.position == nil then db.position = {}; db.position.point = "CENTER"; db.position.x, db.position.y = 0, 0 end
	self:SetPoint(db.position.point, db.position.x, db.position.y)
	if db.lock == nil then db.lock = true end
	self:SetMovable(not db.lock)
end
local function SlashShow()
	db.hide = false
	Update()
end
local function SlashHide(self)
	db.hide = true
	self:Hide()
end
local function SlashToggle()
	if db.hide then
		SlashShow()
	else
		SlashHide(ArtifactPowerUserButton)
	end
end
LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject(addonName, {
	type = "launcher",
	icon = C_ArtifactUI.GetEquippedArtifactInfo() and GetItemIcon(C_ArtifactUI.GetEquippedArtifactInfo()) or "Interface\\Icons\\INV_Staff_2h_ArtifactAegwynsStaff_D_01",
	OnClick = function(clickedframe, button)
		SlashToggle()
	end,
})
local function SlashFunction(msg)
	local self = ArtifactPowerUserButton
	msg = string.lower(msg)
	local command, param = msg:match("^(%S*)%s*(.-)$")
	if command == "show" then
		SlashShow()
	elseif command == "hide" then
		SlashHide(self)
	elseif command == "toggle" then
		SlashToggle()
	elseif command == "size" then
		local size = tonumber(param)
		if size then
			db.size = size
			self:SetSize(size, size)
			print("ArtifactPowerUser size = "..size)
		else
			print("Format /apu size 64")
		end
	elseif command == "lock" then
		db.lock = true
		self:SetMovable(false)
		print("ArtifactPowerUser locked")
	elseif command == "unlock" then
		db.lock = false
		self:SetMovable(true)	
		print("ArtifactPowerUser unlocked")
	elseif command == "reset" then
		db.position.point = "CENTER"; db.position.x, db.position.y = 0, 0
		self:SetPoint(db.position.point, db.position.x, db.position.y)
	else
		print("Possible commands: show, hide, toggle, size, lock, unlock, reset")
	end	
end

function ArtifactPowerUserButton_OnLoad(self)
	self:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	self:RegisterForDrag("LeftButton")
	
	self:RegisterEvent("ADDON_LOADED")
	self:RegisterEvent("BAG_UPDATE_DELAYED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	
	SLASH_ARTIFACTPOWERUSER1 = "/apu"
	SlashCmdList["ARTIFACTPOWERUSER"] = SlashFunction
	_G["BINDING_NAME_CLICK ArtifactPowerUserButton:LeftButton"] = "ArtifactPowerUse"
	BINDING_HEADER_ARTIFACTPOWERUSER = "ArtifactPowerUser"
end
function ArtifactPowerUserButton_OnEvent(self, event, ...)
	if event == "ADDON_LOADED" and (...) == addonName then
		LoadSettings(self)
	elseif event == "BAG_UPDATE_DELAYED" then
		Update()
	elseif event == "PLAYER_REGEN_DISABLED" then
		self:Hide()
		self:UnregisterEvent("BAG_UPDATE_DELAYED")
	elseif event == "PLAYER_REGEN_ENABLED" then
		Update()
		self:RegisterEvent("BAG_UPDATE_DELAYED")
	end
end
function ArtifactPowerUserButton_OnShow(self)
	Update()
end
function ArtifactPowerUserButton_OnHide(self)
	self:SetAttribute("type", nil)
	self:SetAttribute("item", nil)
end
function ArtifactPowerUserButton_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetHyperlink(itemLink)
end
function ArtifactPowerUserButton_OnLeave(self)
	GameTooltip_Hide()
end
function ArtifactPowerUserButton_OnDragStart(self)
	if self:IsMovable() then
		self:StartMoving()
	end
end
function ArtifactPowerUserButton_OnReceiveDrag(self)
	self:StopMovingOrSizing()
	db.position.point, _, _, db.position.x, db.position.y = self:GetPoint("CENTER")
end
